{% load static from staticfiles %}
<div id="fb-root"></div>
<!--Some facebook code requires the fb-root div -->
<script src="{% static "django_facebook/js/facebook.js"%}" type="text/javascript"></script>
<script>
    facebookAppId = '{{ FACEBOOK_APP_ID }}';
    facebookDefaultScope = {{ FACEBOOK_DEFAULT_SCOPE_JS }};
    staticUrl = '{{ STATIC_URL }}';
    function facebookJSLoaded(){
        FB.init({appId: facebookAppId, status: false, cookie: true, xfbml: true, oauth: true});
    }
    window.fbAsyncInit = facebookJSLoaded;
    F = new facebookClass(facebookAppId);
    F.load();
</script>
