<html>
    <head>
        <script>
            var oauth_url = '{{ location|safe }}';
            window.top.location = oauth_url;
        </script>
    </head>
    <body>
    	{% if show_body %}
        Redirecting you to <a href="{{ location }}">{{ location }}</a>
        {% endif %}
    </body>
</html>