__author__ = 'Thierry Schellenbach'
__copyright__ = 'Copyright 2010, Thierry Schellenbach'
__credits__ = ['Thierry Schellenbach, mellowmorning.com, @tschellenbach']


__license__ = 'BSD'
__version__ = '6.0.6'
__maintainer__ = 'Thierry Schellenbach'
__email__ = 'thierryschellenbach@gmail.com'
__status__ = 'Production'

'''
Some links which help me with publishing this code

rest editor
http://rst.ninjs.org/

updating pypi
python setup.py sdist upload
http://pypi.python.org/pypi

setting up pip for editing
http://www.pip-installer.org/en/latest/index.html
pip install -e ./
'''
