import json


def json_to_python(json_string):
    '''
    I receive error mails encoded as json, this function converts the data into
    easy usable python dicts and pprint them
    '''

    python_dict = json.loads(json_string)
    return python_dict

if __name__ == '__main__':
    json_string = r'''{"website": "http://fernandaferrervazquez.blogspot.com/\r\nhttp://twitter.com/fferrervazquez\r\nhttp://comunidad.redfashion.es/profile/fernandaferrervazquez\r\nhttp://www.facebook.com/group.php?gid3D40257259997&ref3Dts\r\nhttp://fernandaferrervazquez.spaces.live.com/blog/cns!EDCBAC31EE9D9A0C!326.trak\r\nhttp://www.linkedin.com/myprofile?trk3Dhb_pro\r\nhttp://www.youtube.com/account#profile\r\nhttp://www.flickr.com/\r\n Mi galer\u00eda\r\nhttp://www.flickr.com/photos/wwwfernandaferrervazquez-showroomrecoletacom/ \r\n\r\nhttp://www.facebook.com/pages/Buenos-Aires-Argentina/Fernanda-F-Showroom-Recoleta/200218353804?ref3Dts\r\nhttp://fernandaferrervazquez.wordpress.com/wp-admin/", "last_name": "Ferrer Vazquez", "locale": "es_LA", "hometown": {"id": "109238842431095", "name": "La Coru\u00f1a, Galicia, Spain"}, "image": "https://graph.facebook.com/me/picture?type3Dlarge&access_token3D100314226679773|2.1ZkFz1Cusu5RlY1xGft_Pg__.86400.1303534800.4-1157872766|ZgsRrRYwp2pqHEHtncqS5-rBiSg"
    , "timezone": -3, "education": [{"school": {"id": "115805661780893", "name": "uba arquitectura dise\u00f1o y urbanismo"}, "classes": [{"description": "direccion de arte", "id": "188729917815843", "name": "AAAP"}, {"with": [{"id": "546455670", "name": "Gustavo Lento"}], "id": "200142039998266", "name": "Indumentaria"}, {"description": "dise\u00f1o ", "with": [{"id": "1285841477", "name": "La Maison Madrid"}], "id": "102056769875267", "name": "figurin"}, {"id": "194463713907701", "name": "indumentaria -figurin-seriado"}, {"with": [{"id": "704003068", "name": "Mariano Toledo"}], "id": "180792848632395", "name": "indumentaria-figurin -seriado.etc"}], "type": "College", "concentration": [{"id": "176664472378719", "name": "indumentaria y dise\u00f1o textil"}]}, {"school": {"id": "107827585907242", "name": "Miami International University of Art & Design"}, "classes": [{"id": "116042535136769", "name": "whadrove asesory"}], "type": "Graduate School", "concentration": [{"id": "146715355391868", "name": "producer-  stylist -  asesora de imagen -personal-shooper -whadrove asesory"}]}, {"school": {"id": "105548239478812", "name": "New York Institute of Technology"}, "classes": [{"id": "199608053396849", "name": "Reporter, Producer and Director"}, {"id": "209828275699578", "name": "personal shooper"}, {"id": "208913322452336", "name": "Designer-stylisti"}], "type": "College", "concentration": [{"id": "184808144897752", "name": "whadrobing"}]}, {"school": {"id": "106431826058746", "name": "Colegio Nuestra Se\u00f1ora de la Misericordia"}, "type": "High School"}, {"school": {"id": "106011862771707", "name": "Mary E. Graham"}, "type": "High School"}, {"school": {"id": "106011862771707", "name": "Mary E. Graham"}, "type": "High School"}], "id": "1157872766", "first_name": "Fernanda", "verified": true, "languages": [{"id": "111021838921932", "name": "Espa\u00f1ol"}, {"id": "110867825605119", "name": "Ingles"}, {"id": "108083115891989", "name": "Portugu\u00e9s"}, {"id": "113051562042989", "name": "Italiano"}], "location": {"id": "106423786059675", "name": "Buenos Aires, Argentina"}, "image_thumb": "https://graph.facebook.com/me/picture?access_token3D100314226679773|2.1ZkFz1Cusu5RlY1xGft_Pg__.86400.1303534800.4-1157872766|ZgsRrRYwp2pqHEHtncqS5-rBiSg", "email": "fer-fer-666@hotmail.com", "username": "FernandaFerrerVazquez", "bio": "ATELIER : TODOS LOS DIAS DE 14 A 22 HS C CITA PREVIA. MIX DE ESTETICAS - ECLECTICA - PRENDAS UNICAS-HAND MADE- SEASSON LESS RETROMODERNIDAD -CUSTOMIZADO !!!\r\n\r\nMe encanta todo el proceso que lleva lograr un objeto que sea unico....personalizarlo de acuerdo a cada uno...a cada persona....a su forma de ser.......elegir los materiales, los avios, las telas...siendo la premisa DIFERENCIARSE!!!!\r\n Atemporal..Retro modernidad...Eclecticismo!!!", "birthday": "04/09/1973", "link": "http://www.facebook.com/FernandaFerrerVazquez", "name": "Fernanda Ferrer Vazquez", "gender": "female", "work": [{"description": "ATELIER : un mix de esteticas que la hacen eclectica, multicultural, excentrica, customizada, artesanal, unica , exclusiva, lujosa, eco-reciclada,con bordados a mano y texturas..especial atencion a los peque\u00f1os detalles!!!! \nSeason less \nHand made ", "employer": {"id": "113960871952651", "name": "fernanda ferrer vazquez"}, "location": {"id": "106423786059675", "name": "Buenos Aires, Argentina"}, "position": {"id": "145270362165639", "name": "Dise\u00f1adora"}, "start_date": "2008-12", "projects": [{"description": "produccion-estilismo-fotografia-make up-pelo-asesoramiento de imagen-whadrobing-perso\u00f1an shooper-rent an outfit!!", "id": "200218353804", "name": "Fernanda F -Showroom Recoleta"}]}, {"employer": {"id": "198034683561689", "name": "el atelier de la isla"}}], "updated_time": "2011-04-22T04:28:27+0000"}'''
    json_to_python(json_string)
