#facebook_shade {
    position            : fixed;
    height              : 100%;
    width               : 100%;
    top                 : 0px;
    left                : 0px;
    background-color    : #222;
    opacity             : 0.8;
    filter              : alpha(opacity=80);
    -moz-opacity        : 0.8;
    z-index             : 1000;
}

#facebook_lightbox {
    position            : relative;
    width               : 300px; 
    background-color    : #fff; 
    margin              : 200px auto; 
    text-align          : center; 
    padding             : 30px 20px;        
    -moz-box-shadow     : 0px 0px 8px #000;
    -webkit-box-shadow  : 0px 0px 8px #000;
    box-shadow          : 0px 0px 8px #000;    
    moz-border-radius   : 2px;
    border-radius       : 2px;
    z-index             : 1001;
}

#facebook_wrapper {
    position            : absolute;
    top                 : 0px;
    width               : 100%;
    z-index             : 1001;
}

#facebook_loading {
    margin-top          : 15px;
}

#facebook_close {
    position            : absolute;
    right               : 10px;
    top                 : 10px;
    width               : 16px;
    height              : 16px;
    background          : url('../images/facebook_close.png');
    cursor              : pointer;
}
